<?php

namespace Drupal\minikanban_agent;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Component\Uuid\UuidInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Database\Query\SelectInterface;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\minikanban_agent\DTO\AsyncJobData;
use Drupal\minikanban_agent\Enum\AsyncJobStatus;

/**
 * Manager for async jobs.
 */
readonly class AsyncJobManager {

  public const TABLE = 'kanban_agent_async_job';

  public function __construct(
    private Connection $database,
    private UuidInterface $uuid,
    private AsyncJobPluginManager $plugin_manager,
    private EntityTypeManagerInterface $entityTypeManager,
    private AccountProxyInterface $current_user,
    private DateFormatterInterface $dateFormatter,
    private TimeInterface $time,
  ) {}

  /**
   * Load the record of a job.
   *
   * @param string $uuid
   *   The job UUID.
   *
   * @return \Drupal\minikanban_agent\DTO\AsyncJobData|null
   *   The job data.
   */
  public function loadJob(string $uuid): ?AsyncJobData {
    return $this->database
      ->select(self::TABLE, 'j')
      ->fields('j', array_keys(get_class_vars(AsyncJobData::class)))
      ->condition('uuid', $uuid)
      ->range(0, 1)
      ->execute()
      ->fetchAll(\PDO::FETCH_FUNC, [AsyncJobData::class, 'fromPdo'])[0] ?? NULL;
  }

  /**
   * Load the record of a job.
   *
   * @param int|null $task
   *   The task entity ID, or NULL to not filter by task.
   * @param callable|null $callback
   *   A callback to modify the query, e.g. add status filter or sort.
   *
   * @return \Drupal\minikanban_agent\DTO\AsyncJobData[]]
   *   The job data.
   */
  public function loadJobs(?int $task = NULL, ?callable $callback = NULL): array {
    $query = $this->database
      ->select(self::TABLE, 'j')
      ->fields('j', array_keys(get_class_vars(AsyncJobData::class)));
    if ($task) {
      $query->condition('task', $task);
    }
    if ($callback) {
      $callback($query);
    }
    return $query
      ->execute()
      ->fetchAll(\PDO::FETCH_FUNC, [AsyncJobData::class, 'fromPdo']);
  }

  /**
   * Get the next job in the queue to check.
   *
   * @return \Drupal\minikanban_agent\DTO\AsyncJobData|null
   *   The next job to check, or NULL if there are none.
   */
  public function loadNextJob(): ?AsyncJobData {
    return $this->loadJobs(
      callback: AsyncJobManager::nextIncompleteJobFilter(...),
    )[0] ?? NULL;
  }

  /**
   * Get a count of jobs.
   *
   * @param int|null $task
   *   The task entity ID, or NULL to not filter by task.
   * @param callable|null $callback
   *   A callback to modify the query, e.g. add status filter or sort.
   *
   * @return int
   *   The number of matching jobs.
   */
  public function countJobs(?int $task = NULL, ?callable $callback = NULL): int {
    $query = $this->database
      ->select(self::TABLE, 'j')
      ->fields('j');
    if ($task) {
      $query->condition('task', $task);
    }
    if ($callback) {
      $callback($query);
    }
    return (int) $query->countQuery()->execute()->fetchField();
  }

  /**
   * Create a record of a job.
   *
   * @param \Drupal\minikanban_agent\DTO\AsyncJobData $job
   *   The job.
   */
  public function saveJob(AsyncJobData $job): void {
    $job->uuid ??= $this->uuid->generate();
    $this->database
      ->upsert(self::TABLE)
      ->fields($job->toDbRecord())
      ->key('uuid')
      ->execute();
  }

  /**
   * Check for an update on a job, progressing the task if finished.
   *
   * @param \Drupal\minikanban_agent\DTO\AsyncJobData $job
   *   The job data.
   */
  public function checkForUpdate(AsyncJobData $job): void {
    /** @var \Drupal\minikanban\Entity\Task|null $task */
    $task = $this->entityTypeManager
      ->getStorage('kanban_task')
      ->load($job->task);
    if (!$task) {
      return;
    }

    // @todo Error handling for jobs.
    $this->plugin_manager
      ->createInstance($job->plugin)
      ->checkForUpdate($job, $task);

    $job->last_checked = $this->time->getCurrentTime();
    $this->saveJob($job);

    if ($job->status === AsyncJobStatus::Completed) {
      $task->addComment(
        "Async job {$job->label} ({$job->uuid}) completed at {$this->dateFormatter->format($job->completed, 'short')}:\n{$job->description}",
        $this->current_user->id(),
      );

      // Check for other incomplete jobs if the status is blocked.
      if ($task->get('status')->value === 'blocked') {
        $pending_jobs = $this->countJobs(
          $job->task,
          self::incompleteJobFilter(...),
        );
        if ($pending_jobs === 0) {
          $task->addComment(
            'All background tasks have finished, task unblocked.',
            $this->current_user->id(),
          );
          $task
            ->set('status', 'todo')
            ->save();
        }
      }
    }
    elseif ($job->status === AsyncJobStatus::Failed) {
      $task
        ->set('assigned', $task->getEpic()->getOwner())
        ->set('status', 'blocked')
        ->save();
      $task->addComment(
        "Async job {$job->label} ({$job->uuid}) failed:\n{$job->description}",
        $this->current_user->id(),
      );
    }
  }

  /**
   * Filter callback for incomplete jobs.
   *
   * @param \Drupal\Core\Database\Query\SelectInterface $query
   *   The query.
   */
  public static function incompleteJobFilter(SelectInterface $query): void {
    $query->condition(
      'status',
      [
        AsyncJobStatus::Completed->value,
        AsyncJobStatus::Failed->value,
      ],
      'NOT IN',
    );
  }

  /**
   * Filter callback for incomplete jobs.
   *
   * @param \Drupal\Core\Database\Query\SelectInterface $query
   *   The query.
   */
  public static function nextIncompleteJobFilter(SelectInterface $query): void {
    self::incompleteJobFilter($query);
    // Do not poll individual jobs more than every 15 seconds.
    $query->condition(
      $query->orConditionGroup()
        ->condition(
          'last_checked',
          \Drupal::time()->getCurrentTime() - 15,
          '<=',
        )
        ->isNull('last_checked'),
    );
    $query->orderBy('last_checked');
    $query->orderBy('created');
    $query->range(0, 1);
  }

}
