<?php

declare(strict_types=1);

namespace Drupal\minikanban_agent\Command;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Session\AccountSwitcherInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\minikanban_agent\AgentHelper;
use Drupal\minikanban_agent\AsyncJobManager;
use Drupal\minikanban_agent\MinikanbanRunner;
use Symfony\Component\Console\Attribute\AsCommand;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Command\SignalableCommandInterface;
use Symfony\Component\Console\Exception\InvalidOptionException;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;

#[AsCommand(
  name: 'minikanban-agent:run',
  description: 'Run the Minikanban Agent worker.',
  aliases: ['mia', 'minikanban:agent'],
)]
final class MinikanbanRunnerCommand extends Command implements SignalableCommandInterface {

  use StringTranslationTrait;

  /**
   * The command input.
   */
  protected InputInterface $input;

  /**
   * The command output.
   */
  protected OutputInterface $output;

  /**
   * Whether a stop signal has been received.
   */
  private bool $stop = FALSE;

  /**
   * Constructs a RunWorkerAgent object.
   */
  public function __construct(
    private readonly MinikanbanRunner $runner,
    private readonly AccountSwitcherInterface $accountSwitcher,
    private readonly EntityTypeManagerInterface $entityTypeManager,
    private readonly AgentHelper $minikanbanAgentHelper,
    private readonly AsyncJobManager $asyncJobManager,
    private readonly TimeInterface $time,
  ) {
    parent::__construct();
  }

  /**
   * {@inheritdoc}
   */
  protected function configure() {
    parent::configure();
    $this->addOption(
      'cooldown',
      'c',
      InputOption::VALUE_REQUIRED,
      'Cooldown time in seconds',
    );
    $this->addOption(
      'min-run',
      NULL,
      InputOption::VALUE_REQUIRED,
      'Minimum run time',
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function initialize(InputInterface $input, OutputInterface $output): void {
    parent::initialize($input, $output);

    $cooldown = $input->getOption('cooldown');
    if ($cooldown !== NULL) {
      if (!filter_var($cooldown, FILTER_VALIDATE_INT) || $cooldown <= 0) {
        throw new InvalidOptionException('--cooldown must be positive integer.');
      }
    }

    $min_run = $input->getOption('min-run');
    if ($min_run !== NULL) {
      if (!filter_var($min_run, FILTER_VALIDATE_INT) || $min_run <= 0) {
        throw new InvalidOptionException('--min-run must be positive integer.');
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getSubscribedSignals(): array {
    return [\SIGTERM, \SIGQUIT, \SIGINT];
  }

  /**
   * {@inheritdoc}
   */
  public function handleSignal(int $signal): false|int {
    $this->stop = TRUE;
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  protected function execute(InputInterface $input, OutputInterface $output): int {
    $this->input = $input;
    $this->output = $output;

    // If an interrupt was caught, stop gracefully.
    if ($this->shouldStop()) {
      return self::SUCCESS;
    }

    $this->accountSwitcher->switchTo(
      $this->entityTypeManager
        ->getStorage('user')
        ->load($this->minikanbanAgentHelper->getAiAgentId()),
    );

    $start = $this->time->getCurrentTime();
    $end = $start + ($input->getOption('min-run') ?: 0);

    try {
      do {
        if ($this->generateTasks()) {
          continue;
        }

        // If an interrupt was caught, stop gracefully.
        if ($this->shouldStop()) {
          return self::SUCCESS;
        }

        if ($this->executeTask()) {
          continue;
        }

        // If an interrupt was caught, stop gracefully.
        if ($this->shouldStop()) {
          return self::SUCCESS;
        }

        if ($this->checkAsyncJob()) {
          continue;
        }

        // If an interrupt was caught, stop gracefully.
        if ($this->shouldStop()) {
          return self::SUCCESS;
        }

        // We didn't generate or execute tasks, so cooldown if requested.
        if ($cooldown = $input->getOption('cooldown')) {
          $output->writeln((string) $this->t(
            'No tasks or workers found, cooling down @duration',
            [
              '@duration' => $this->formatPlural($cooldown, '@count second', '@count seconds'),
            ],
          ));
          for ($i = 1; $i <= $cooldown; $i++) {
            // If an interrupt was caught, stop gracefully.
            if ($this->shouldStop()) {
              return self::SUCCESS;
            }
            sleep(1);
          }
        }
      } while ($this->time->getCurrentTime() < $end);

      return self::SUCCESS;
    }
    finally {
      $this->accountSwitcher->switchBack();
    }
  }

  /**
   * Ask the project manager to generate tasks.
   *
   * @return bool
   *   Whether we attempted to generate tasks for an epic.
   */
  public function generateTasks(): bool {
    $this->output->writeln('The project manager is looking for epics to generate tasks for.');

    // @todo consider adding a lock mechanism.
    $epic = $this->runner->getNextEpicInQueue();
    if (!$epic) {
      $this->output->writeln('No epics found.');
      return FALSE;
    }

    $this->output->writeln("Starting to progress/generate tasks for epic '{$epic->label()}' ({$epic->id()}).");
    $this->runner->generateTasksForEpic($epic);

    return TRUE;
  }

  /**
   * Runs one job for one work agent if available.
   *
   * @return bool
   *   If a worker was run.
   */
  public function executeTask(): bool {
    $this->output->writeln('Our workers are looking for tasks to start.');

    // @todo consider adding a lock mechanism.
    $task = $this->runner->getNextTaskInQueue();
    if (!$task) {
      $this->output->writeln('No tasks found.');
      return FALSE;
    }

    $this->output->writeln("Starting to execute task '{$task->label()}' ({$task->id()}).");
    $this->runner->executeTask($task);

    return TRUE;
  }

  /**
   * Runs one async check if available.
   *
   * @return bool
   *   If an async check was run.
   */
  private function checkAsyncJob(): bool {
    $this->output->writeln('Checking for async job updates.');

    // @todo consider adding a lock mechanism.
    $job = $this->asyncJobManager->loadNextJob();
    if (!$job) {
      $this->output->writeln('No async jobs found.');
      return FALSE;
    }

    $this->output->writeln("Starting to check job {$job->label} ({$job->uuid}).");
    $this->asyncJobManager->checkForUpdate($job);
    $this->output->writeln("[{$job->status->value}] {$job->description}");

    return TRUE;
  }

  /**
   * Whether we should stop execution due to an interrupt signal.
   *
   * @return bool
   *   TRUE if we whouls stop.
   *
   * @phpstan-impure
   */
  private function shouldStop(): bool {
    return $this->stop;
  }

}
