<?php

namespace Drupal\minikanban_agent\DTO;

use Drupal\minikanban_agent\Enum\AsyncJobStatus;

/**
 * DTO for kanban async jobs.
 */
final class AsyncJobData {

  /**
   * Construct the job data.
   *
   * @param int $task
   *   The task entity ID.
   * @param string $plugin
   *   The plugin that handles this job.
   * @param string $label
   *   The label for the job.
   * @param string $remote_id
   *   The remote identifier for the job.
   * @param \Drupal\minikanban_agent\Enum\AsyncJobStatus $status
   *   The status for the job.
   * @param int $created
   *   The timestamp the job was created at.
   * @param array<string, mixed> $metadata
   *   Metadata for the job.
   * @param ?int $progress
   *   The current progress of the job, or NULL if there is no progress.
   * @param string|null $description
   *   The status description.
   * @param string|null $uuid
   *   The UUID for the job.
   * @param int|null $completed
   *   The timestamp the job was completed at, or NULL if not completed.
   * @param int|null $last_checked
   *   The timestamp the job was last checked at, or NULL if not completed.
   */
  public function __construct(
    public int $task,
    public string $plugin,
    public string $label,
    public string $remote_id,
    public AsyncJobStatus $status,
    public int $created,
    public ?array $metadata = NULL,
    public ?int $progress = NULL,
    public ?string $description = NULL,
    public ?string $uuid = NULL,
    public ?int $completed = NULL,
    public ?int $last_checked = NULL,
  ) {
  }

  /**
   * PDO fetch record into DTO.
   *
   * @param array<string, scalar> ...$props
   *   The props from the record.
   *
   * @return self
   *   The job DTO.
   */
  public static function fromPdo(...$props): self {
    $props = array_combine(
      array_keys(get_class_vars(self::class)),
      $props,
    );
    $props['status'] = AsyncJobStatus::from($props['status']);
    if (isset($props['metadata'])) {
      $props['metadata'] = unserialize(
        $props['metadata'],
        ['allowed_classes' => FALSE],
      );
    }
    return new self(...$props);
  }

  /**
   * Convert to a database record.
   *
   * @return array<string, scalar>
   *   The array for the database record.
   */
  public function toDbRecord(): array {
    $values = get_object_vars($this);
    $values['status'] = $this->status->value;
    if (isset($values['metadata'])) {
      $values['metadata'] = serialize($values['metadata']);
    }
    return $values;
  }

}
