<?php

declare(strict_types=1);

namespace Drupal\minikanban_agent\Hook;

use Drupal\ai_agents\AiAgentInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Hook\Attribute\Hook;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Alter the ai_agent_form.
 */
class AiAgentFormAlter {

  use StringTranslationTrait;

  public function __construct(
    private readonly RequestStack $requestStack,
  ) {}

  /**
   * Alter the suggested tokens.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   * @param string $form_id
   *   The form ID.
   */
  #[Hook('form_ai_agent_form_alter')]
  public function alterTokens(array &$form, FormStateInterface $form_state, string $form_id): void {
    if (isset($form['prompt_detail']['token_help'])) {
      $form['prompt_detail']['token_help']['#token_types'][] = 'kanban_project';
      $form['prompt_detail']['token_help']['#token_types'][] = 'kanban_epic';
      $form['prompt_detail']['token_help']['#token_types'][] = 'kanban_task';
    }
  }

  /**
   * Add the minikanban agent configuration.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   * @param string $form_id
   *   The form ID.
   */
  #[Hook('form_ai_agent_form_alter')]
  public function addMinikanbanAgentConfig(array &$form, FormStateInterface $form_state, string $form_id): void {
    $entity = $form_state->getFormObject()->getEntity();
    assert($entity instanceof AiAgentInterface);

    $form['minikanban_agent'] = [
      '#type' => 'details',
      '#title' => $this->t('Minikanban configuration'),
      '#tree' => TRUE,
    ];
    $form['minikanban_agent']['completion_status_prompt'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Completion status prompt'),
      '#description' => $this->t('The prompt used to guide the agent in determining the resulting status on completion of execution. If not provided, a default prompt will be used.'),
      '#default_value' => $entity->getThirdPartySetting('minikanban_agent', 'completion_status_prompt'),
    ];
    $form['#entity_builders'][] = [self::class, 'minikanbanAgentConfigEntityBuilder'];
  }

  /**
   * Entity builder for the minikanban agent config.
   *
   * @param string $entity_type_id
   *   The entity type ID.
   * @param \Drupal\ai_agents\AiAgentInterface $entity
   *   The agent entity.
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public static function minikanbanAgentConfigEntityBuilder(
    string $entity_type_id,
    AiAgentInterface $entity,
    array $form,
    FormStateInterface $form_state,
  ): void {
    $entity->setThirdPartySetting(
      'minikanban_agent',
      'completion_status_prompt',
      $form_state->getValue(['minikanban_agent', 'completion_status_prompt']),
    );
  }

  /**
   * Set the form default when adding a swarm.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   * @param string $form_id
   *   The form ID.
   */
  #[Hook('form_ai_agent_add_form_alter')]
  public function setSwarmDefault(array &$form, FormStateInterface $form_state, string $form_id): void {
    if ($this->requestStack->getCurrentRequest()->query->get('is_swarm')) {
      $form['orchestration_agent']['#default_value'] = TRUE;
    }
  }

}
