<?php

declare(strict_types=1);

namespace Drupal\minikanban_agent\Hook;

use Drupal\ai\AiProviderPluginManager;
use Drupal\ai\OperationType\Chat\ChatInput;
use Drupal\ai\OperationType\Chat\ChatMessage;
use Drupal\ai\Service\PromptCodeBlockExtractor\PromptCodeBlockExtractorInterface;
use Drupal\Core\Hook\Attribute\Hook;
use Drupal\minikanban\CommentInterface;
use Drupal\minikanban\TaskInterface;
use Drupal\minikanban_agent\AgentHelper;
use Symfony\Component\DependencyInjection\Attribute\Autowire;

/**
 * Task entity CRUD hooks.
 */
readonly class TaskCrud {

  public function __construct(
    private AgentHelper $minikanbanAgentHelper,
    #[Autowire('@ai.provider')]
    private AiProviderPluginManager $aiProvider,
    #[Autowire('@ai.prompt_code_block_extractor')]
    private PromptCodeBlockExtractorInterface $promptCodeBlockExtractor,
  ) {}

  /**
   * Handle transferring context from completed tasks to it's blockers.
   *
   * @param \Drupal\minikanban\TaskInterface $task
   *   The task entity.
   */
  #[Hook('kanban_task_update')]
  public function copyContextBetweenTasks(TaskInterface $task): void {
    // Don't operate on newly completed tasks.
    if ($this->isNewlyDone($task)) {
      return;
    }

    // Ignore tasks that aren't assigned to the AI agent.
    if (!$this->isAssignedToAiAgent($task)) {
      return;
    }

    $blocked_tasks = $task->getBlockingTasks();
    if (!$blocked_tasks) {
      return;
    }

    $comment = array_map(
      fn (CommentInterface $comment) => $comment->comment->value,
      $task->getComments(),
    );
    array_unshift($comment, 'A blocking ticket was completed and has context that this ticket needs to finish. See the results below:');
    $comment = implode('<br/>', $comment);

    // Add a comment to each blocked tasks.
    foreach ($blocked_tasks as $blocked_task) {
      $blocked_task->addComment($comment, $this->minikanbanAgentHelper->getAiAgentId());
    }
  }

  /**
   * Mark the epic as in progress when a task is started.
   *
   * @param \Drupal\minikanban\TaskInterface $task
   *   The task entity.
   */
  #[Hook('kanban_task_update')]
  public function startEpic(TaskInterface $task): void {
    // Ignore new tasks.
    if ($task->isNew()) {
      return;
    }

    // Ignore tasks that aren't assigned to the AI agent.
    if (!$this->isAssignedToAiAgent($task)) {
      return;
    }

    // Check the task is in progress.
    if ($task->status->value !== 'in_progress') {
      return;
    }

    // Check we have a todo epic.
    $epic = $task->getEpic();
    if (!$epic || $epic->status->value !== 'todo') {
      return;
    }

    // Set the epic to in progress.
    $epic->set('status', 'in_progress')->save();
  }

  /**
   * Mark the epic for review if all tasks are done.
   *
   * @param \Drupal\minikanban\TaskInterface $task
   *   The task entity.
   */
  #[Hook('kanban_task_update')]
  public function completeEpic(TaskInterface $task): void {
    // Don't operate on new tasks, or not newly completed.
    if ($task->isNew() || $this->isNewlyDone($task)) {
      return;
    }

    // Ignore tasks that aren't assigned to the AI agent.
    if (!$this->isAssignedToAiAgent($task)) {
      return;
    }

    // Check we have an in progress epic.
    $epic = $task->getEpic();
    if (!$epic || $epic->status->value !== 'in_progress') {
      return;
    }

    // Check that all tasks are done.
    $epic_tasks = $epic->getTasks();
    foreach ($epic_tasks as $epic_task) {
      if ($epic_task->status->value !== 'done') {
        return;
      }
    }

    // If all tasks are done, we move the context and set the epic to in_review.
    $comment = 'All tasks are done, the following results hopefully fits what you were looking for. If not please check tasks where it went wrong and rewrite a new epic.<br /><br />';
    foreach ($epic_tasks as $epic_task) {
      $commments = $epic_task->getComments();
      foreach ($commments as $comment_entity) {
        $comment .= $comment_entity->comment->value;
        $comment .= "<br />";
      }
    }

    // Rewrite the comment using an LLM.
    // @todo Replace this with an agent.
    // @todo Consider moving this to a queue/async process.
    $defaults = $this->aiProvider->getDefaultProviderForOperationType('chat');
    $llm = $this->aiProvider->createInstance($defaults['provider_id']);
    $input = new ChatInput([
      new ChatMessage("user", "Look at the following results from different agents and try to summarize the results into a single comment. Please respond with HTML in an html code block, using a, em, strong, ul, ol, li, img tags where needed. The results are:<br /><br />$comment"),
    ]);
    $response = $llm->chat($input, $defaults['model_id']);
    $html = $this->promptCodeBlockExtractor->extract($response->getNormalized(), 'html');

    // Add comment.
    $epic->addComment($html, $this->minikanbanAgentHelper->getAiAgentId());
    $epic->status = 'in_review';
    $epic->save();
  }

  /**
   * Check if a task is newly done.
   *
   * @param \Drupal\minikanban\TaskInterface $task
   *   The task entity.
   *
   * @return bool
   *   TRUE if the task is newly done.
   */
  private function isNewlyDone(TaskInterface $task): bool {
    return $task->status->value !== 'done' || $task->original?->status->value === 'done';
  }

  /**
   * Check if a task is assigned the the AI agent user.
   *
   * @param \Drupal\minikanban\TaskInterface $task
   *   The task entity.
   *
   * @return bool
   *   TRUE if the task is assigned to the AI agent user.
   */
  private function isAssignedToAiAgent(TaskInterface $task): bool {
    return $task->assigned->target_id == $this->minikanbanAgentHelper->getAiAgentId();
  }

}
