<?php

declare(strict_types=1);

namespace Drupal\minikanban_agent\Hook;

use Drupal\ai_agents\AiAgentInterface as AiAgentEntityInterface;
use Drupal\ai_agents\PluginManager\AiAgentManager;
use Drupal\Core\Hook\Attribute\Hook;
use Drupal\Core\Render\BubbleableMetadata;
use Drupal\Core\Render\Markup;
use Drupal\Core\Serialization\Yaml;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Symfony\Component\DependencyInjection\Attribute\Autowire;

/**
 * Token hooks for minikanban agent.
 */
readonly class Tokens {

  public function __construct(
    #[Autowire('@plugin.manager.ai_agents')]
    private AiAgentManager $aiAgentManager,
  ) {}

  /**
   * Get token information.
   *
   * @phpstan-return array{tokens: array<string, array<string, array<string, string|\Stringable>>>}
   */
  #[Hook('token_info')]
  public function info(): array {
    return [
      'tokens' => [
        'ai_agent' => [
          'minikanban_sub_agents' => [
            'name' => new TranslatableMarkup('Sub agents'),
            'description' => new TranslatableMarkup('Agents available to this for Minikanban task management.'),
          ],
        ],
      ],
    ];
  }

  /**
   * Tokens for replacement.
   *
   * @see \hook_tokens()
   *
   * @phpstan-param string $type
   * @phpstan-param array<string, string> $tokens
   * @phpstan-param array<string, mixed> $data
   * @phpstan-param array<string, mixed> $options
   * @phpstan-param \Drupal\Core\Render\BubbleableMetadata $bubbleable_metadata
   *
   * @phpstan-return array<string, string>
   */
  #[Hook('tokens')]
  public function tokens(
    string $type,
    array $tokens,
    array $data,
    array $options,
    BubbleableMetadata $bubbleable_metadata,
  ): array {
    $replacements = [];

    if ($type === 'ai_agent' && !empty($data['ai_agent'])) {
      $agent = $data['ai_agent'];
      assert($agent instanceof AiAgentEntityInterface);

      foreach ($tokens as $name => $original) {
        switch ($name) {
          case 'minikanban_sub_agents':
            $sub_agents = $this->aiAgentManager->getDefinitions();

            // If there are any sub-agent tools selected, restrict the list.
            $selected_sub_agents = array_intersect(
              array_filter($agent->get('tools')),
              array_keys($sub_agents),
            );
            if ($selected_sub_agents) {
              $sub_agents = array_intersect_key(
                $sub_agents,
                array_fill_keys($selected_sub_agents, TRUE),
              );
            }

            $replacements[$original] = Markup::create(Yaml::encode(array_values(array_map(
              fn (array $definition) => [
                'ID' => $definition['id'],
                'Name' => (string) $definition['label'],
              ],
              $sub_agents,
            ))));
            break;
        }
      }
    }

    return $replacements;
  }

}
