<?php

namespace Drupal\minikanban_agent;

use Drupal\ai\AiProviderPluginManager;
use Drupal\ai\OperationType\Chat\ChatMessage;
use Drupal\ai_agents\PluginBase\AiAgentEntityWrapper;
use Drupal\ai_agents\PluginInterfaces\AiAgentInterface;
use Drupal\ai_agents\PluginInterfaces\ConfigAiAgentInterface;
use Drupal\ai_agents\PluginManager\AiAgentManager;
use Drupal\ai_agents\Task\Task;
use Drupal\Component\Plugin\Exception\PluginException;
use Drupal\Component\Render\FormattableMarkup;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\Serialization\Yaml;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Utility\Error;
use Drupal\minikanban\EpicInterface;
use Drupal\minikanban\TaskInterface;
use Drupal\minikanban_agent\Exceptions\AgentRunnerException;
use Drupal\minikanban_agent\Exceptions\FaultyAgentException;
use Drupal\minikanban_agent\Exceptions\TaskStatusUpdateException;
use Symfony\Component\DependencyInjection\Attribute\Autowire;

/**
 * Task generator.
 */
class MinikanbanRunner {

  public function __construct(
    #[Autowire('@plugin.manager.ai_agents')]
    private readonly AiAgentManager $agentManager,
    #[Autowire('@ai.provider')]
    private readonly AiProviderPluginManager $aiProviderPluginManager,
    private readonly EntityTypeManagerInterface $entityTypeManager,
    private readonly AccountProxyInterface $currentUser,
    private readonly AgentHelper $minikanbanAgentHelper,
    #[Autowire('@logger.channel.minikanban_agent')]
    private readonly LoggerChannelInterface $logger,
  ) {
  }

  /**
   * Get the next epic in the queue.
   *
   * @return \Drupal\minikanban\EpicInterface|null
   *   The next epic in the queue, or NULL if there are no epics.
   */
  public function getNextEpicInQueue(): ?EpicInterface {
    $storage = $this->entityTypeManager
      ->getStorage('kanban_epic');

    // Query for the next epic to work on.
    $result = $storage
      ->getQuery()
      ->condition('status', 'todo')
      ->condition('assigned', $this->minikanbanAgentHelper->getAiAgentId())
      ->accessCheck()
      ->sort('changed')
      ->range(0, 1)
      ->execute();

    // Return the result.
    return $result ? $storage->load(reset($result)) : NULL;
  }

  /**
   * Generate tasks from epic.
   *
   * @param \Drupal\minikanban\EpicInterface $epic
   *   The epic.
   */
  public function generateTasksForEpic(EpicInterface $epic): void {
    // Move into in progress so we don't get multiple processes.
    $epic->set('status', 'in_progress')->save();

    $agent = $this->setupAiAgentForEntity($epic);

    // Add context where relevant.
    if ($agent instanceof ConfigAiAgentInterface) {
      $agent->setTokenContexts([
        'kanban_project' => $epic->getProject(),
        'kanban_epic' => $epic,
      ]);
    }

    try {
      $solvability = $agent->determineSolvability();
      if ($solvability === AiAgentInterface::JOB_NOT_SOLVABLE) {
        throw new AgentRunnerException('The agent was not able to plan tasks in the allowed loops.');
      }

      $solvability = $this->determineNewStatus($epic, $agent);
      if ($solvability === AiAgentInterface::JOB_NOT_SOLVABLE) {
        throw new AgentRunnerException('The agent was not able to give a status update in the allowed loops.');
      }
    }
    catch (TaskStatusUpdateException $task_update) {
      $epic
        ->set('status', $task_update->status)
        ->save();
    }
    catch (\Exception $exception) {
      $epic
        ->set('assigned', $epic->getOwner())
        ->set('status', 'blocked')
        ->save();
      $epic->addComment(
        "Error: {$exception->getMessage()}",
        $this->currentUser->id(),
      );
      throw $exception;
    }
  }

  /**
   * Get the next task in the queue.
   *
   * @return \Drupal\minikanban\TaskInterface|null
   *   The next task in the queue, or NULL if there are no tasks.
   */
  public function getNextTaskInQueue(): ?TaskInterface {
    $storage = $this->entityTypeManager
      ->getStorage('kanban_task');

    // Query for the next task to work on.
    $result = $storage->getQuery()
      ->condition('ai_agent', '', '<>')
      ->condition('status', 'todo')
      ->condition('assigned', $this->minikanbanAgentHelper->getAiAgentId())
      ->accessCheck()
      ->sort('changed')
      ->range(0, 1)
      ->execute();

    // Return the result.
    return $result ? $storage->load(reset($result)) : NULL;
  }

  /**
   * Execute a task.
   *
   * @param \Drupal\minikanban\TaskInterface $task_entity
   *   The task entity.
   */
  public function executeTask(TaskInterface $task_entity): void {
    $this->logger->debug('Starting task %label [@id]', [
      '%label' => $task_entity->label(),
      '@id' => $task_entity->id(),
    ]);
    $task_entity->set('status', 'in_progress')->save();

    $agent = $this->setupAiAgentForEntity($task_entity);

    // Add context where relevant.
    if ($agent instanceof ConfigAiAgentInterface) {
      $agent->setTokenContexts([
        'kanban_project' => $task_entity->getProject(),
        'kanban_epic' => $task_entity->getEpic(),
        'kanban_task' => $task_entity,
      ]);
    }

    try {
      $solvability = $agent->determineSolvability();
      if ($solvability === AiAgentInterface::JOB_NOT_SOLVABLE) {
        throw new AgentRunnerException('The agent was not able to solve the problem in the allowed loops.');
      }

      $solvability = $this->determineNewStatus($task_entity, $agent);
      if ($solvability === AiAgentInterface::JOB_NOT_SOLVABLE) {
        throw new AgentRunnerException('The agent was not able to give a status update in the allowed loops.');
      }

      $this->logger->debug('Finished task %label [@id] with status %status', [
        '%label' => $task_entity->label(),
        '@id' => $task_entity->id(),
        '%status' => $task_entity->get('status')->value,
      ]);
    }
    catch (TaskStatusUpdateException $task_update) {
      $task_entity
        ->set('status', $task_update->status)
        ->save();
      $task_entity->addComment(
        $task_update->getMessage(),
        $this->currentUser->id(),
      );

      $this->logger->debug('Finished task %label [@id] with status %status', [
        '%label' => $task_entity->label(),
        '@id' => $task_entity->id(),
        '%status' => $task_entity->get('status')->value,
      ]);
    }
    catch (\Throwable $exception) {
      $task_entity
        ->set('assigned', $task_entity->getEpic()->getOwner())
        ->set('status', 'blocked')
        ->save();
      $task_entity->addComment(
        "Error: {$exception->getMessage()}",
        $this->currentUser->id(),
      );

      Error::logException(
        $this->logger,
        $exception,
        "Error executing task %label [@id]\n" . Error::DEFAULT_ERROR_MESSAGE,
        [
          '%label' => $task_entity->label(),
          '@id' => $task_entity->id(),
        ]
      );

      throw $exception;
    }
  }

  /**
   * Set up the AI agent and task for a kanban epic/task entity.
   *
   * @param \Drupal\minikanban\EpicInterface|\Drupal\minikanban\TaskInterface $entity
   *   The epic/task entity.
   *
   * @return \Drupal\ai_agents\PluginInterfaces\AiAgentInterface
   *   The configured AI agent.
   */
  private function setupAiAgentForEntity(EpicInterface|TaskInterface $entity): AiAgentInterface {
    // Get the agent.
    if ($entity instanceof EpicInterface) {
      $project = $entity->getProject();

      // Get the orchestration agent.
      $agent_id = $entity->get('ai_agent')->target_id
        ?? $project->get('ai_agent')->target_id
        ?? 'minikanban_generic';
    }
    else {
      $agent_id = $entity->get('ai_agent')->target_id;
      if (!$agent_id) {
        throw new FaultyAgentException('No task agent set',);
      }
    }

    // Load the agent.
    /** @var \Drupal\ai_agents\PluginInterfaces\AiAgentInterface $agent */
    try {
      $agent = $this->agentManager->createInstance($agent_id);
    }
    catch (PluginException $exception) {
      throw new FaultyAgentException(
        "Task agent {$agent_id} not found.",
        previous: $exception,
      );
    }

    // Get the LLM.
    $defaults = $this->aiProviderPluginManager->getDefaultProviderForOperationType('chat_with_tools');
    $provider = $this->aiProviderPluginManager->createInstance($defaults['provider_id']);
    $agent->setAiProvider($provider);
    $agent->setModelName($defaults['model_id']);
    $agent->setAiConfiguration([]);
    $agent->setCreateDirectly(TRUE);

    // Set up the AI task.
    $agent_task = new Task($entity->get('description')->value);
    $agent_task->setTitle($entity->label());

    $agent_task->setFiles($entity->getAttachedFiles());

    $comments = [];
    foreach ($entity->getComments() as $comment) {
      $comments[] = [
        'role' => $comment->get('uid')->entity?->getDisplayName() ?? new FormattableMarkup('Anonymous @id', ['@id' => $comment->get('uid')->target_id ?? 0]),
        'message' => $comment->get('comment')->value,
      ];
    }
    $agent_task->setComments($comments);

    $agent->setTask($agent_task);

    return $agent;
  }

  /**
   * Re-run the agent with the task status function call.
   *
   * @param \Drupal\minikanban\EpicInterface|\Drupal\minikanban\TaskInterface $entity
   *   The epic/task we want the status for.
   * @param \Drupal\ai_agents\PluginInterfaces\AiAgentInterface $agent
   *   The agent being executed.
   *
   * @return int
   *   The solvability.
   */
  private function determineNewStatus(EpicInterface|TaskInterface $entity, AiAgentInterface $agent): int {
    if (!$agent instanceof AiAgentEntityWrapper) {
      throw new AgentRunnerException("The agent cannot be re-run for the status of task.");
    }

    // Override the functions so we only set solvability.
    if ($entity instanceof EpicInterface) {
      $function_call_id = 'minikanban_epic_update';
      $entity_context_name = 'kanban_epic';
      $allowed_statuses = ['blocked', 'in_progress'];
    }
    else {
      $function_call_id = 'minikanban_task_update';
      $entity_context_name = 'kanban_task';
      $allowed_statuses = ['blocked', 'done'];
    }

    $agent->overrideFunctions([
      'tools' => [
        $function_call_id => TRUE,
      ],
      'tool_settings' => [
        $function_call_id => [
          'return_directly' => TRUE,
        ],
      ],
      'tool_usage_limits' => [
        $function_call_id => [
          'kanban_project' => [
            'action' => 'force_value',
            'hide_property' => TRUE,
            'values' => [(int) $entity->getProject()->id()],
          ],
          $entity_context_name => [
            'action' => 'force_value',
            'hide_property' => TRUE,
            'values' => [(int) $entity->id()],
          ],
          'status' => [
            'action' => 'only_allow',
            'hide_property' => FALSE,
            'values' => $allowed_statuses,
          ],
        ],
      ],
    ]);

    // Add our solvability prompt to the chat.
    $chat_history = $agent->getChatHistory();

    // Some LLMs get confused by having chat tools in the history that they no
    // longer have access to (looking at you Claude...). So prevent the LLM
    // concluding the task is blocked because the tool is no longer available,
    // scrub the tool history. We keep the message content itself, which
    // the output of the tool.
    /** @var \Drupal\ai\OperationType\Chat\ChatMessage $item */
    foreach ($chat_history as $item) {
      // If the LLM uses the specific tool role, we need to switch that to user
      // to prevent validation errors (e.g. OpenAI).
      if ($item->getRole() === 'tool') {
        $item->setRole('user');
      }
      // The assistant may have not given a message, which will produce an
      // error. To prevent this and give additional context, we'll list the tool
      // calls.
      elseif ($item->getRole() === 'assistant') {
        $tools = $item->getTools() ?? [];
        if ($tools) {
          $content = $item->getText();
          $content .= ($content ? "\n\n" : '') . "Tools called:";
          foreach ($tools as $tool) {
            $content .= "\n" . Yaml::encode($tool->getOutputRenderArray());
          }
          $item->setText($content);
        }
      }

      $item->setToolsId('');
      $item->setTools([]);
    }

    $completion_prompt = NULL;
    if ($agent instanceof AiAgentEntityWrapper) {
      $completion_prompt = trim($agent->getAiAgentEntity()->getThirdPartySetting('minikanban_agent', 'completion_status_prompt'));
    }
    $completion_prompt = $completion_prompt ?: 'If you encountered any errors, the status should be blocked. Otherwise the status should be done.';
    $chat_history[] = new ChatMessage(
      'user',
      "Disregard previous instructions as you have finished your attempt to complete the task. They simply provide context as to what you have already done. You do not need to re-run any activity from the task as that has all been finished and the chat history has the results. I have removed the previous tool list used to complete the task as they are no longer relevant.\n\nPlease use the {$function_call_id} tool to report back the final status of this task. Do not provide a text response, only the correctly structured tool call. Include a comment explaining the status change, including making a note of the status you are setting this task to.\n\n{$completion_prompt}",
    );
    $agent->setChatHistory($chat_history);

    // Run, reset, and return.
    $solvability = $agent->determineSolvability();
    $agent->resetFunctions();
    return $solvability;
  }

}
