<?php

namespace Drupal\minikanban_agent\Plugin\AiFunctionCall;

use Drupal\ai\Attribute\FunctionCall;
use Drupal\ai\Base\FunctionCallBase;
use Drupal\ai\Service\FunctionCalling\ExecutableFunctionCallInterface;
use Drupal\ai\Service\FunctionCalling\FunctionCallInterface;
use Drupal\ai\Utility\ContextDefinitionNormalizer;
use Drupal\ai_agents\PluginInterfaces\AiAgentContextInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\Context\ContextDefinition;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\minikanban_agent\AgentHelper;
use Symfony\Component\DependencyInjection\ContainerInterface;

#[FunctionCall(
  id: 'minikanban_epic_create',
  function_name: 'minikanban_epic_create',
  name: 'Create a kanban epic',
  description: 'Create a kanban epic for the given project.',
  group: 'minikanban',
  context_definitions: [
    'kanban_project' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup('Kanban Project'),
      required: TRUE,
      description: new TranslatableMarkup('The Kanban Project entity ID.'),
    ),
    'agent' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup('Orchestration agent'),
      required: FALSE,
      description: new TranslatableMarkup('The ID of the orchestration agent to assign the epic to. Omit to use the default.'),
    ),
    'title' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup('Title'),
      required: TRUE,
      description: new TranslatableMarkup('The epic title, maximum 80 characters.'),
    ),
    'description' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup('Description'),
      required: TRUE,
      description: new TranslatableMarkup('A description of the epic that is enough for the orchestration agent to plan the required work.'),
    ),
    'status' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup('Status'),
      required: TRUE,
      description: new TranslatableMarkup('The new status to set.'),
    ),
    'background_color' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup('Background color'),
      required: TRUE,
      description: new TranslatableMarkup("A hex color code, including the # prefix, used to highlight the epic and it's tasks."),
    ),
  ]
)]
class KanbanEpicCreate extends FunctionCallBase implements ExecutableFunctionCallInterface, AiAgentContextInterface {

  /**
   * The entity type manager.
   */
  private EntityTypeManagerInterface $entityTypeManager;

  /**
   * The minikanban agent helper service.
   */
  private AgentHelper $minikanbanAgentHelper;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): FunctionCallInterface|static {
    $instance = new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      new ContextDefinitionNormalizer(),
    );
    $instance->entityTypeManager = $container->get('entity_type.manager');
    $instance->minikanbanAgentHelper = $container->get(AgentHelper::class);
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function execute(): void {
    $project_id = $this->getContextValue('kanban_project');
    $agent = $this->getContextValue('agent');
    $title = $this->getContextValue('title');
    $description = $this->getContextValue('description');
    $status = $this->getContextValue('status');
    $background_color = $this->getContextValue('background_color');

    $epic = $this->entityTypeManager
      ->getStorage('kanban_epic')
      ->create([
        'project' => $project_id,
        'label' => $title,
        'description' => $description,
        'status' => $status,
        'ai_agent' => $agent,
        'assigned' => $agent ? $this->minikanbanAgentHelper->getAiAgentId() : NULL,
        'background_color' => $background_color,
      ]);
    $epic->save();
    $this->setOutput("Created as '{$epic->label()}' with ID {$epic->id()} in status {$epic->get('status')->value}");
  }

}
