<?php

namespace Drupal\minikanban_agent\Plugin\AiFunctionCall;

use Drupal\ai\Attribute\FunctionCall;
use Drupal\ai\Base\FunctionCallBase;
use Drupal\ai\Service\FunctionCalling\ExecutableFunctionCallInterface;
use Drupal\ai\Service\FunctionCalling\FunctionCallInterface;
use Drupal\ai\Utility\ContextDefinitionNormalizer;
use Drupal\ai_agents\PluginInterfaces\AiAgentContextInterface;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\Context\ContextDefinition;
use Drupal\Core\Serialization\Yaml;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Symfony\Component\DependencyInjection\ContainerInterface;

#[FunctionCall(
  id: 'minikanban_epic_details',
  function_name: 'minikanban_epic_details',
  name: 'Get epic details',
  description: 'Get the details, including comments, of an epic.',
  group: 'minikanban',
  context_definitions: [
    'kanban_project' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup('Kanban Project'),
      required: TRUE,
      description: new TranslatableMarkup('The Kanban Project entity ID the epic belongs to.'),
    ),
    'kanban_epic' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup('Kanban Epic'),
      required: TRUE,
      description: new TranslatableMarkup('The Kanban Epic entity ID to fetch.'),
    ),
  ],
)]
class KanbanEpicDetails extends FunctionCallBase implements ExecutableFunctionCallInterface, AiAgentContextInterface {

  /**
   * The entity type manager.
   */
  private EntityTypeManagerInterface $entityTypeManager;

  /**
   * The date formatter.
   */
  private DateFormatterInterface $dateFormatter;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): FunctionCallInterface|static {
    $instance = new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      new ContextDefinitionNormalizer(),
    );
    $instance->entityTypeManager = $container->get('entity_type.manager');
    $instance->dateFormatter = $container->get('date.formatter');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function execute(): void {
    $project_id = $this->getContextValue('kanban_project');
    $epic_id = $this->getContextValue('kanban_epic');

    /** @var (\Drupal\minikanban\EpicInterface&\Drupal\minikanban\Entity\Epic)|null $epic */
    $epic = $this->entityTypeManager
      ->getStorage('kanban_epic')
      ->load($epic_id);
    if (!$epic) {
      $this->setOutput('Error: Epic not found.');
      return;
    }
    if ($epic->getProject()->id() != $project_id) {
      $this->setOutput("Error: Epic doesn't belong to the project.");
      return;
    }

    $values = [
      'ID' => $epic_id,
      'Title' => $epic->label(),
      'Status' => $epic->get('status')->value,
      'Description' => $epic->get('description')->value,
      'Comments' => [],
    ];

    /** @var \Drupal\minikanban\CommentInterface&\Drupal\minikanban\Entity\Comment $comment */
    foreach ($epic->getComments() as $comment) {
      $values['Comments'][] = [
        'Time' => $this->dateFormatter->format($comment->get('created')->value),
        'Author' => $comment->getAuthorsUsername(),
        'Comment' => (string) $comment->get('comment')->processed,
      ];
    }

    $this->setOutput(Yaml::encode($values));
  }

}
