<?php

namespace Drupal\minikanban_agent\Plugin\AiFunctionCall;

use Drupal\ai\Attribute\FunctionCall;
use Drupal\ai\Base\FunctionCallBase;
use Drupal\ai\Service\FunctionCalling\ExecutableFunctionCallInterface;
use Drupal\ai\Service\FunctionCalling\FunctionCallInterface;
use Drupal\ai\Utility\ContextDefinitionNormalizer;
use Drupal\ai_agents\PluginInterfaces\AiAgentContextInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\Context\ContextDefinition;
use Drupal\Core\Serialization\Yaml;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\minikanban\EpicInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

#[FunctionCall(
  id: 'minikanban_epic_list',
  function_name: 'minikanban_epic_list',
  name: 'List kanban epics',
  description: 'Get a list of Kanban epics for an project.',
  group: 'minikanban',
  context_definitions: [
    'kanban_project' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup('Kanban Project'),
      required: TRUE,
      description: new TranslatableMarkup('The Kanban Project entity ID to get tasks for.'),
    ),
    'status' => new ContextDefinition(
      data_type: 'list',
      label: new TranslatableMarkup('Status'),
      required: FALSE,
      description: new TranslatableMarkup('Optionally filter on task status.'),
    ),
    'assigned_to' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup('Assigned to User'),
      required: FALSE,
      description: new TranslatableMarkup('Optionally filter on assigned to a User entity.'),
    ),
  ],
)]
class KanbanEpicList extends FunctionCallBase implements ExecutableFunctionCallInterface, AiAgentContextInterface {

  /**
   * The entity type manager.
   */
  private EntityTypeManagerInterface $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): FunctionCallInterface|static {
    $instance = new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      new ContextDefinitionNormalizer(),
    );
    $instance->entityTypeManager = $container->get('entity_type.manager');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function execute(): void {
    $project_id = $this->getContextValue('kanban_project');
    $status = $this->getContextValue('status');
    $assigned_to = $this->getContextValue('assigned_to');

    $storage = $this->entityTypeManager
      ->getStorage('kanban_epic');

    $query = $storage
      ->getQuery()
      ->accessCheck(TRUE)
      ->condition('project', $project_id);

    if ($status) {
      $query->condition('status', $status, 'IN');
    }
    if ($assigned_to) {
      $query->condition('assigned', $assigned_to);
    }

    // Load the items, mapping them to a response.
    $items = array_map(
      fn (EpicInterface $task) => array_filter([
        'ID' => (int) $task->id(),
        'Title' => $task->label(),
        'Status' => count($status) !== 1 ? $task->get('status')->value : NULL,
      ]),
      $storage->loadMultiple($query->execute()),
    );

    if (empty($items)) {
      $this->setOutput('No epics found matching the conditions.');
    }
    else {
      $this->setOutput(Yaml::encode(array_values($items)));
    }
  }

}
