<?php

namespace Drupal\minikanban_agent\Plugin\AiFunctionCall;

use Drupal\ai\Attribute\FunctionCall;
use Drupal\ai\Base\FunctionCallBase;
use Drupal\ai\Service\FunctionCalling\ExecutableFunctionCallInterface;
use Drupal\ai\Service\FunctionCalling\FunctionCallInterface;
use Drupal\ai\Utility\ContextDefinitionNormalizer;
use Drupal\ai_agents\PluginInterfaces\AiAgentContextInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\Context\ContextDefinition;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Symfony\Component\DependencyInjection\ContainerInterface;

#[FunctionCall(
  id: 'minikanban_epic_update',
  function_name: 'minikanban_epic_update',
  name: 'Update a kanban epic',
  description: 'Change the kanban epic status, and add comments.',
  group: 'minikanban',
  context_definitions: [
    'kanban_project' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup('Kanban Project'),
      required: TRUE,
      description: new TranslatableMarkup('The Kanban Project entity ID.'),
    ),
    'kanban_epic' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup('Kanban Epic'),
      required: TRUE,
      description: new TranslatableMarkup('The Kanban Epic entity ID.'),
    ),
    'status' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup('Status'),
      required: FALSE,
      description: new TranslatableMarkup('The new status to set.'),
    ),
    'comment' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup('Comment'),
      required: FALSE,
      description: new TranslatableMarkup('A comment to add to the epic.'),
    ),
  ]
)]
class KanbanEpicUpdate extends FunctionCallBase implements ExecutableFunctionCallInterface, AiAgentContextInterface {

  /**
   * The entity type manager.
   */
  private EntityTypeManagerInterface $entityTypeManager;

  /**
   * The current user.
   */
  private AccountProxyInterface $currentUser;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): FunctionCallInterface|static {
    $instance = new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      new ContextDefinitionNormalizer(),
    );
    $instance->entityTypeManager = $container->get('entity_type.manager');
    $instance->currentUser = $container->get('current_user');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function execute(): void {
    $project_id = $this->getContextValue('kanban_project');
    $task_id = $this->getContextValue('kanban_epic');
    $status = $this->getContextValue('status');
    $comment = $this->getContextValue('comment');

    if (!$status && !$comment) {
      $this->setOutput('Warning: No changes requested for the epic.');
      return;
    }

    // Load the epic, check access, and ensure it is part of this project.
    // @todo Move methods onto interface so we can hint against that.
    /** @var \Drupal\minikanban\Entity\Epic|null $epic */
    $epic = $this->entityTypeManager
      ->getStorage('kanban_epic')
      ->load($task_id);
    if (!$epic) {
      $this->setOutput('Error: Unable to load epic.');
      return;
    }
    if (!$epic->access('update')) {
      $this->setOutput('Error: Access denied to epic.');
      return;
    }
    if ($epic->get('project')->target_id != $project_id) {
      $this->setOutput('Error: Epic is not part of this project.');
      return;
    }

    // Update the status.
    if ($status) {
      $epic->setStatus($status);
    }

    if ($comment) {
      $epic->addComment($comment, $this->currentUser->id());
    }

    $epic->save();

    $this->setOutput("'{$epic->label()}' with ID {$epic->id()} updated");
  }

}
