<?php

namespace Drupal\minikanban_agent\Plugin\AiFunctionCall;

use Drupal\ai\Attribute\FunctionCall;
use Drupal\ai\Base\FunctionCallBase;
use Drupal\ai\Service\FunctionCalling\ExecutableFunctionCallInterface;
use Drupal\ai\Service\FunctionCalling\FunctionCallInterface;
use Drupal\ai\Utility\ContextDefinitionNormalizer;
use Drupal\ai_agents\PluginInterfaces\AiAgentContextInterface;
use Drupal\ai_agents\PluginManager\AiAgentManager;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\Context\ContextDefinition;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\minikanban\TaskInterface;
use Drupal\minikanban_agent\AgentHelper;
use Symfony\Component\DependencyInjection\ContainerInterface;

#[FunctionCall(
  id: 'minikanban_task_create',
  function_name: 'minikanban_task_create',
  name: 'Create a kanban task',
  description: 'Create a kanban task for the given project and epic.',
  group: 'minikanban',
  context_definitions: [
    'kanban_project' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup('Kanban Project'),
      required: TRUE,
      description: new TranslatableMarkup('The Kanban Project entity ID.'),
    ),
    'kanban_epic' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup('Kanban Epic'),
      required: TRUE,
      description: new TranslatableMarkup('The Kanban Epic entity ID.'),
    ),
    'agent' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup('Agent'),
      required: FALSE,
      description: new TranslatableMarkup('The ID of the agent to assign the task to. Omit to assign this to a human.'),
    ),
    'title' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup('Title'),
      required: TRUE,
      description: new TranslatableMarkup('The task title, maximum 80 characters.'),
    ),
    'description' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup('Description'),
      required: TRUE,
      description: new TranslatableMarkup('A description of the task that is enough for the worker to fulfill there task.'),
    ),
    'status' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup('Status'),
      required: TRUE,
      description: new TranslatableMarkup('The new status to set.'),
    ),
    'blockers' => new ContextDefinition(
      data_type: 'list',
      label: new TranslatableMarkup('Blockers'),
      required: FALSE,
      description: new TranslatableMarkup('A list of Kanban Task IDs that block this task.'),
    ),
  ]
)]
class KanbanTaskCreate extends FunctionCallBase implements ExecutableFunctionCallInterface, AiAgentContextInterface {

  /**
   * The entity type manager.
   */
  private EntityTypeManagerInterface $entityTypeManager;

  /**
   * The AI agent plugin manager.
   */
  private AiAgentManager $aiAgentManager;

  /**
   * The minikanban agent helper service.
   */
  private AgentHelper $minikanbanAgentHelper;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): FunctionCallInterface|static {
    $instance = new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      new ContextDefinitionNormalizer(),
    );
    $instance->entityTypeManager = $container->get('entity_type.manager');
    $instance->aiAgentManager = $container->get('plugin.manager.ai_agents');
    $instance->minikanbanAgentHelper = $container->get(AgentHelper::class);
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function execute(): void {
    $project_id = $this->getContextValue('kanban_project');
    $epic_id = $this->getContextValue('kanban_epic');
    $agent = $this->getContextValue('agent');
    $title = $this->getContextValue('title');
    $description = $this->getContextValue('description');
    $status = $this->getContextValue('status');
    $blockers = $this->getContextValue('blockers');

    // Check that the agent is valid.
    // @todo See if we can move this into context so the LLM gets hints.
    // @todo Restrict this to the allowed types for the calling.
    if ($agent) {
      if (!$this->aiAgentManager->hasDefinition($agent)) {
        $this->setOutput("Error: {$agent} is not a valid agent.");
        return;
      }
    }

    // @todo Move methods onto interface so we can hint against that.
    /** @var \Drupal\minikanban\Entity\Task $task */
    $task = $this->entityTypeManager
      ->getStorage('kanban_task')
      ->create([
        'project' => $project_id,
        'epic' => $epic_id,
        'label' => $title,
        'description' => $description,
        'status' => $status,
        'ai_agent' => $agent,
        'blockers' => $blockers,
        'assigned' => $agent ? $this->minikanbanAgentHelper->getAiAgentId() : NULL,
      ]);

    // Status todo should be switched to blocked if there are incomplete
    // blocking tasks.
    if ($status === 'todo') {
      $current_blockers = array_filter(
        $task->getBlockers(),
        fn (TaskInterface $blocker) => $blocker->get('status')->value !== 'done',
      );
      if ($current_blockers) {
        $task->setStatus('blocked');
      }
    }

    $task->save();
    $this->setOutput("Created as '{$task->label()}' with ID {$task->id()} in status {$task->get('status')->value}");
  }

}
