<?php

namespace Drupal\minikanban_agent\Plugin\AiFunctionCall;

use Drupal\ai\Attribute\FunctionCall;
use Drupal\ai\Base\FunctionCallBase;
use Drupal\ai\Service\FunctionCalling\ExecutableFunctionCallInterface;
use Drupal\ai\Service\FunctionCalling\FunctionCallInterface;
use Drupal\ai\Utility\ContextDefinitionNormalizer;
use Drupal\ai_agents\PluginInterfaces\AiAgentContextInterface;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\Context\ContextDefinition;
use Drupal\Core\Serialization\Yaml;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Symfony\Component\DependencyInjection\ContainerInterface;

#[FunctionCall(
  id: 'minikanban_task_details',
  function_name: 'minikanban_task_details',
  name: 'Get task details',
  description: 'Get the details, including comments, of an task.',
  group: 'minikanban',
  context_definitions: [
    'kanban_project' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup('Kanban Project'),
      required: TRUE,
      description: new TranslatableMarkup('The Kanban Project entity ID the task belongs to.'),
    ),
    'kanban_task' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup('Kanban Task'),
      required: TRUE,
      description: new TranslatableMarkup('The Kanban Task entity ID to fetch.'),
    ),
  ],
)]
class KanbanTaskDetails extends FunctionCallBase implements ExecutableFunctionCallInterface, AiAgentContextInterface {

  /**
   * The entity type manager.
   */
  private EntityTypeManagerInterface $entityTypeManager;

  /**
   * The date formatter.
   */
  private DateFormatterInterface $dateFormatter;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): FunctionCallInterface|static {
    $instance = new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      new ContextDefinitionNormalizer(),
    );
    $instance->entityTypeManager = $container->get('entity_type.manager');
    $instance->dateFormatter = $container->get('date.formatter');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function execute(): void {
    $project_id = $this->getContextValue('kanban_project');
    $task_id = $this->getContextValue('kanban_task');

    /** @var (\Drupal\minikanban\TaskInterface&\Drupal\minikanban\Entity\Task)|null $task */
    $task = $this->entityTypeManager
      ->getStorage('kanban_task')
      ->load($task_id);
    if (!$task) {
      $this->setOutput('Error: Task not found.');
      return;
    }
    if ($task->getProject()->id() != $project_id) {
      $this->setOutput("Error: Task doesn't belong to the project.");
      return;
    }

    $values = [
      'ID' => $task_id,
      'Title' => $task->label(),
      'Status' => $task->get('status')->value,
      'Description' => $task->get('description')->value,
      'Comments' => [],
    ];

    /** @var \Drupal\minikanban\CommentInterface&\Drupal\minikanban\Entity\Comment $comment */
    foreach ($task->getComments() as $comment) {
      $values['Comments'][] = [
        'Time' => $this->dateFormatter->format($comment->get('created')->value),
        'Author' => $comment->getAuthorsUsername(),
        'Comment' => (string) $comment->get('comment')->processed,
      ];
    }

    $this->setOutput(Yaml::encode($values));
  }

}
