<?php

namespace Drupal\minikanban_agent\Plugin\AiFunctionCall;

use Drupal\ai\Attribute\FunctionCall;
use Drupal\ai\Base\FunctionCallBase;
use Drupal\ai\Service\FunctionCalling\ExecutableFunctionCallInterface;
use Drupal\ai\Service\FunctionCalling\FunctionCallInterface;
use Drupal\ai\Utility\ContextDefinitionNormalizer;
use Drupal\ai_agents\PluginInterfaces\AiAgentContextInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\Context\ContextDefinition;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\minikanban\TaskInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

#[FunctionCall(
  id: 'minikanban_task_update',
  function_name: 'minikanban_task_update',
  name: 'Update a kanban task',
  description: 'Change the kanban task status, add blockers, and add comments.',
  group: 'minikanban',
  context_definitions: [
    'kanban_project' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup('Kanban Project'),
      required: TRUE,
      description: new TranslatableMarkup('The Kanban Project entity ID.'),
    ),
    'kanban_task' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup('Kanban Task'),
      required: TRUE,
      description: new TranslatableMarkup('The Kanban Task entity ID.'),
    ),
    'status' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup('Status'),
      required: FALSE,
      description: new TranslatableMarkup('The new status to set.'),
    ),
    'blockers' => new ContextDefinition(
      data_type: 'list',
      label: new TranslatableMarkup('Blockers'),
      required: FALSE,
      description: new TranslatableMarkup('A list of Kanban Task IDs that block this task.'),
    ),
    'comment' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup('Comment'),
      required: FALSE,
      description: new TranslatableMarkup('A comment to add to the task.'),
    ),
  ]
)]
class KanbanTaskUpdate extends FunctionCallBase implements ExecutableFunctionCallInterface, AiAgentContextInterface {

  /**
   * The entity type manager.
   */
  private EntityTypeManagerInterface $entityTypeManager;

  /**
   * The current user.
   */
  private AccountProxyInterface $currentUser;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): FunctionCallInterface|static {
    $instance = new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      new ContextDefinitionNormalizer(),
    );
    $instance->entityTypeManager = $container->get('entity_type.manager');
    $instance->currentUser = $container->get('current_user');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function execute(): void {
    $project_id = $this->getContextValue('kanban_project');
    $task_id = $this->getContextValue('kanban_task');
    $status = $this->getContextValue('status');
    $blockers = $this->getContextValue('blockers');
    $comment = $this->getContextValue('comment');

    if (!$status && !$blockers && !$comment) {
      $this->setOutput('Warning: No changes requested for the task.');
      return;
    }

    // Load the task, check access, and ensure it is part of this project.
    // @todo Move methods onto interface so we can hint against that.
    /** @var \Drupal\minikanban\Entity\Task|null $task */
    $task = $this->entityTypeManager
      ->getStorage('kanban_task')
      ->load($task_id);
    if (!$task) {
      $this->setOutput('Error: Unable to load task.');
      return;
    }
    if (!$task->access('update')) {
      $this->setOutput('Error: Access denied to task.');
      return;
    }
    if ($task->get('project')->target_id != $project_id) {
      $this->setOutput('Error: Task is not part of this project.');
      return;
    }

    // Set any new blockers.
    if ($blockers) {
      $existing_blockers = array_map(
        fn (array $item) => $item['target_id'],
        $task->get('blockers')->getValue(),
      );
      $task->set('blockers', array_unique(array_merge($existing_blockers, $blockers)));
    }

    // Update the status.
    if ($status) {
      $task->setStatus($status);

      // Status todo should be switched to blocked if there are incomplete
      // blocking tasks.
      if ($status === 'todo') {
        $current_blockers = array_filter(
          $task->getBlockers(),
          fn (TaskInterface $blocker) => $blocker->get('status')->value !== 'done',
        );
        if ($current_blockers) {
          $task->setStatus('blocked');
        }
      }
    }

    if ($comment) {
      $task->addComment($comment, $this->currentUser->id());
    }

    $task->save();

    $this->setOutput("'{$task->label()}' with ID {$task->id()} updated and in status {$task->get('status')->value}");
  }

}
